package com.jiuqi.syntax;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.Calendar;
import java.util.Date;
import java.util.Hashtable;
import java.util.Map;

/* loaded from: com_jiuqi_syntax.jar:com/jiuqi/syntax/AutoFunctionListener.class */
public final class AutoFunctionListener extends CustomFunctionListener {
    private Object provider;
    private String classify;
    private Map funcMap;
    private static final int SY_FUNC_CALL_ERROR = 10400;
    private static final int SY_FUNC_RET_ERROR = 10401;
    private static final int SY_FUNC_ACL_ERROR = 10402;

    public AutoFunctionListener(Object provider) {
        this(provider, null);
    }

    public AutoFunctionListener(Object provider, String classify) {
        this.provider = provider;
        this.classify = classify;
        initFunctions();
    }

    private void initFunctions() {
        this.funcMap = new Hashtable();
        if (this.provider == null) {
            return;
        }
        Class pc = this.provider.getClass();
        Method[] methods = pc.getDeclaredMethods();
        for (int i = 0; i < methods.length; i++) {
            if (Modifier.isPublic(methods[i].getModifiers())) {
                Function func = new Function();
                func.name = methods[i].getName();
                func.resultType = convertType(methods[i].getReturnType());
                if (func.resultType != 0) {
                    func.classify = this.classify;
                    Class[] paramTypes = methods[i].getParameterTypes();
                    int j = 0;
                    while (true) {
                        if (j < paramTypes.length) {
                            FunctionParam param = new FunctionParam();
                            param.name = "p" + Integer.toString(j);
                            param.type = convertType(paramTypes[j]);
                            if (param.type == 0) {
                                break;
                            }
                            param.canOmit = false;
                            param.isSign = false;
                            func.params.add(param);
                            j++;
                        } else {
                            this.functions.add(func);
                            this.funcMap.put(func, methods[i]);
                            break;
                        }
                    }
                }
            }
        }
    }

    private int convertType(Class typeClass) {
        if (Integer.TYPE.equals(typeClass) || Integer.class.equals(typeClass)) {
            return 1;
        }
        if (Double.TYPE.equals(typeClass) || Double.class.equals(typeClass)) {
            return 2;
        }
        if (String.class.equals(typeClass)) {
            return 3;
        }
        if (Boolean.TYPE.equals(typeClass) || Boolean.class.equals(typeClass)) {
            return 5;
        }
        if (Calendar.class.equals(typeClass)) {
            return 6;
        }
        return 0;
    }

    public String getClassify() {
        return this.classify;
    }

    @Override // com.jiuqi.syntax.CustomFunctionListener, com.jiuqi.syntax.FunctionListener
    public int callFunction(String funcName, CommonDatas paramValues, CommonData resultData, boolean isCheckMode) {
        Function func = this.functions.getFunc(funcName);
        if (func == null) {
            return SyntaxParser.SY_FUNC_NO_EXIST;
        }
        Method method = (Method) this.funcMap.get(func);
        try {
            Object[] args = convertParams(paramValues, func);
            Object ret = method.invoke(this.provider, args);
            resultData.type = func.resultType;
            if (!convertObject(ret, resultData)) {
                return SY_FUNC_RET_ERROR;
            }
            return 0;
        } catch (SyntaxException e) {
            return e.getErrorCode();
        } catch (IllegalAccessException unused) {
            return SY_FUNC_ACL_ERROR;
        } catch (IllegalArgumentException unused2) {
            return SyntaxParser.SY_FUNC_BAD_PARAM;
        } catch (InvocationTargetException e2) {
            e2.printStackTrace();
            return SY_FUNC_CALL_ERROR;
        }
    }

    private Object[] convertParams(CommonDatas paramValues, Function func) throws SyntaxException {
        Object[] params = new Object[func.params.size()];
        int i = 0;
        while (i < func.params.size()) {
            CommonData value = paramValues.size() <= i ? null : paramValues.getItem(i);
            if (value == null || value.isNull) {
                params[i] = null;
            } else {
                switch (func.params.getParam(i).type) {
                    case 1:
                        params[i] = new Integer((int) value.getIntRealValue());
                        break;
                    case 2:
                        params[i] = new Double(value.getIntRealValue());
                        break;
                    case 3:
                        params[i] = value.cdString;
                        break;
                    case 4:
                    default:
                        throw new SyntaxException(SyntaxParser.SY_FUNC_BAD_PARAM);
                    case 5:
                        params[i] = new Boolean(value.cdBool);
                        break;
                    case 6:
                        params[i] = value.cdDate;
                        break;
                }
            }
            i++;
        }
        return params;
    }

    private boolean convertObject(Object value, CommonData result) {
        if (value == null) {
            result.isNull = true;
            result.initNullValue();
            return true;
        }
        result.isNull = false;
        switch (result.type) {
            case 1:
                if (value instanceof Integer) {
                    result.cdInt = ((Integer) value).intValue();
                    return true;
                }
                if (value instanceof Double) {
                    result.cdInt = ((Double) value).intValue();
                    return true;
                }
                return false;
            case 2:
                if (value instanceof Integer) {
                    result.cdReal = ((Integer) value).intValue();
                    break;
                } else if (value instanceof Double) {
                    result.cdReal = ((Double) value).doubleValue();
                    break;
                } else {
                    return false;
                }
            case 3:
                break;
            case 4:
            default:
                return false;
            case 5:
                if (value instanceof Boolean) {
                    result.cdBool = ((Boolean) value).booleanValue();
                    return true;
                }
                return false;
            case 6:
                if (value instanceof Calendar) {
                    result.cdDate = (Calendar) value;
                    return true;
                }
                if (value instanceof Date) {
                    result.setDateValue((Date) value);
                    return true;
                }
                if (value instanceof Long) {
                    Calendar date = Calendar.getInstance();
                    date.setTimeInMillis(((Long) value).longValue());
                    result.cdDate = date;
                    return true;
                }
                return false;
        }
        result.cdString = value.toString();
        return true;
    }

    @Override // com.jiuqi.syntax.CustomFunctionListener, com.jiuqi.syntax.FunctionListener
    public String getCodeMessage(int retCode) {
        switch (retCode) {
            case SY_FUNC_CALL_ERROR /* 10400 */:
                return "函数调用时出现程序异常";
            case SY_FUNC_RET_ERROR /* 10401 */:
                return "错误的函数返回值类型";
            case SY_FUNC_ACL_ERROR /* 10402 */:
                return "类或函数无法访问";
            default:
                return null;
        }
    }

    /* JADX INFO: Access modifiers changed from: private */
    /* loaded from: com_jiuqi_syntax.jar:com/jiuqi/syntax/AutoFunctionListener$SyntaxException.class */
    public class SyntaxException extends Exception {
        private int errorCode;

        public SyntaxException(int errorCode) {
            this.errorCode = errorCode;
        }

        public int getErrorCode() {
            return this.errorCode;
        }
    }
}
